# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class TrimmedSolidSplineCutMode(enum.IntEnum):
    """Types of Cut modes used to generate Cartesian grid representing subdomains of trimmed solid spline.
    """
    HYBRID = 0
    """This is faster mode and may defeature the input mesh to represent each subdomain.

    **This is a beta parameter**. **The behavior and name may change in the future**."""
    EXACT = 1
    """This mode guarantees to represent the exact input mesh for each subdomain without any defeaturing.

    **This is a beta parameter**. **The behavior and name may change in the future**."""

class TetMeshSplineParams(CoreObject):
    """Parameters for meshing the solid spline.

    Parameters
    ----------
    model: Model
        Model to create a ``TetMeshSplineParams`` object with default parameters.
    feature_angle: float, optional
        Feature angle used in meshing of the solid spline.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    mode: TrimmedSolidSplineCutMode, optional
        Cut mode to specify rule for mesh cell selection in the volume mesh.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    decimation_factor: float, optional
        Decimation factor used in meshing of the solid spline.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    json_data: dict, optional
        JSON dictionary to create a ``TetMeshSplineParams`` object with provided parameters.

    Examples
    --------
    >>> tet_mesh_spline_params = prime.TetMeshSplineParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            feature_angle: float,
            mode: TrimmedSolidSplineCutMode,
            decimation_factor: float):
        self._feature_angle = feature_angle
        self._mode = TrimmedSolidSplineCutMode(mode)
        self._decimation_factor = decimation_factor

    def __init__(
            self,
            model: CommunicationManager=None,
            feature_angle: float = None,
            mode: TrimmedSolidSplineCutMode = None,
            decimation_factor: float = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``TetMeshSplineParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``TetMeshSplineParams`` object with default parameters.
        feature_angle: float, optional
            Feature angle used in meshing of the solid spline.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        mode: TrimmedSolidSplineCutMode, optional
            Cut mode to specify rule for mesh cell selection in the volume mesh.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        decimation_factor: float, optional
            Decimation factor used in meshing of the solid spline.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        json_data: dict, optional
            JSON dictionary to create a ``TetMeshSplineParams`` object with provided parameters.

        Examples
        --------
        >>> tet_mesh_spline_params = prime.TetMeshSplineParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["featureAngle"] if "featureAngle" in json_data else None,
                TrimmedSolidSplineCutMode(json_data["mode"] if "mode" in json_data else None),
                json_data["decimationFactor"] if "decimationFactor" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [feature_angle, mode, decimation_factor])
            if all_field_specified:
                self.__initialize(
                    feature_angle,
                    mode,
                    decimation_factor)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "TetMeshSplineParams")
                    json_data = param_json["TetMeshSplineParams"] if "TetMeshSplineParams" in param_json else {}
                    self.__initialize(
                        feature_angle if feature_angle is not None else ( TetMeshSplineParams._default_params["feature_angle"] if "feature_angle" in TetMeshSplineParams._default_params else (json_data["featureAngle"] if "featureAngle" in json_data else None)),
                        mode if mode is not None else ( TetMeshSplineParams._default_params["mode"] if "mode" in TetMeshSplineParams._default_params else TrimmedSolidSplineCutMode(json_data["mode"] if "mode" in json_data else None)),
                        decimation_factor if decimation_factor is not None else ( TetMeshSplineParams._default_params["decimation_factor"] if "decimation_factor" in TetMeshSplineParams._default_params else (json_data["decimationFactor"] if "decimationFactor" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            feature_angle: float = None,
            mode: TrimmedSolidSplineCutMode = None,
            decimation_factor: float = None):
        """Set the default values of the ``TetMeshSplineParams`` object.

        Parameters
        ----------
        feature_angle: float, optional
            Feature angle used in meshing of the solid spline.
        mode: TrimmedSolidSplineCutMode, optional
            Cut mode to specify rule for mesh cell selection in the volume mesh.
        decimation_factor: float, optional
            Decimation factor used in meshing of the solid spline.
        """
        args = locals()
        [TetMeshSplineParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``TetMeshSplineParams`` object.

        Examples
        --------
        >>> TetMeshSplineParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in TetMeshSplineParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._feature_angle is not None:
            json_data["featureAngle"] = self._feature_angle
        if self._mode is not None:
            json_data["mode"] = self._mode
        if self._decimation_factor is not None:
            json_data["decimationFactor"] = self._decimation_factor
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "feature_angle :  %s\nmode :  %s\ndecimation_factor :  %s" % (self._feature_angle, self._mode, self._decimation_factor)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def feature_angle(self) -> float:
        """Feature angle used in meshing of the solid spline.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._feature_angle

    @feature_angle.setter
    def feature_angle(self, value: float):
        self._feature_angle = value

    @property
    def mode(self) -> TrimmedSolidSplineCutMode:
        """Cut mode to specify rule for mesh cell selection in the volume mesh.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._mode

    @mode.setter
    def mode(self, value: TrimmedSolidSplineCutMode):
        self._mode = value

    @property
    def decimation_factor(self) -> float:
        """Decimation factor used in meshing of the solid spline.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._decimation_factor

    @decimation_factor.setter
    def decimation_factor(self, value: float):
        self._decimation_factor = value

class RefineTetMeshParams(CoreObject):
    """Parameters for meshing the solid spline.

    Parameters
    ----------
    model: Model
        Model to create a ``RefineTetMeshParams`` object with default parameters.
    nisr: int, optional
        Interpolation elements in the local r-direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    niss: int, optional
        Interpolation elements in the local s-direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    nist: int, optional
        Interpolation elements in the local t-direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    tolerance: float, optional
        Tolerance for boundary refinement.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    json_data: dict, optional
        JSON dictionary to create a ``RefineTetMeshParams`` object with provided parameters.

    Examples
    --------
    >>> refine_tet_mesh_params = prime.RefineTetMeshParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            nisr: int,
            niss: int,
            nist: int,
            tolerance: float):
        self._nisr = nisr
        self._niss = niss
        self._nist = nist
        self._tolerance = tolerance

    def __init__(
            self,
            model: CommunicationManager=None,
            nisr: int = None,
            niss: int = None,
            nist: int = None,
            tolerance: float = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``RefineTetMeshParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``RefineTetMeshParams`` object with default parameters.
        nisr: int, optional
            Interpolation elements in the local r-direction.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        niss: int, optional
            Interpolation elements in the local s-direction.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        nist: int, optional
            Interpolation elements in the local t-direction.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        tolerance: float, optional
            Tolerance for boundary refinement.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        json_data: dict, optional
            JSON dictionary to create a ``RefineTetMeshParams`` object with provided parameters.

        Examples
        --------
        >>> refine_tet_mesh_params = prime.RefineTetMeshParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nisr"] if "nisr" in json_data else None,
                json_data["niss"] if "niss" in json_data else None,
                json_data["nist"] if "nist" in json_data else None,
                json_data["tolerance"] if "tolerance" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [nisr, niss, nist, tolerance])
            if all_field_specified:
                self.__initialize(
                    nisr,
                    niss,
                    nist,
                    tolerance)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "RefineTetMeshParams")
                    json_data = param_json["RefineTetMeshParams"] if "RefineTetMeshParams" in param_json else {}
                    self.__initialize(
                        nisr if nisr is not None else ( RefineTetMeshParams._default_params["nisr"] if "nisr" in RefineTetMeshParams._default_params else (json_data["nisr"] if "nisr" in json_data else None)),
                        niss if niss is not None else ( RefineTetMeshParams._default_params["niss"] if "niss" in RefineTetMeshParams._default_params else (json_data["niss"] if "niss" in json_data else None)),
                        nist if nist is not None else ( RefineTetMeshParams._default_params["nist"] if "nist" in RefineTetMeshParams._default_params else (json_data["nist"] if "nist" in json_data else None)),
                        tolerance if tolerance is not None else ( RefineTetMeshParams._default_params["tolerance"] if "tolerance" in RefineTetMeshParams._default_params else (json_data["tolerance"] if "tolerance" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            nisr: int = None,
            niss: int = None,
            nist: int = None,
            tolerance: float = None):
        """Set the default values of the ``RefineTetMeshParams`` object.

        Parameters
        ----------
        nisr: int, optional
            Interpolation elements in the local r-direction.
        niss: int, optional
            Interpolation elements in the local s-direction.
        nist: int, optional
            Interpolation elements in the local t-direction.
        tolerance: float, optional
            Tolerance for boundary refinement.
        """
        args = locals()
        [RefineTetMeshParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``RefineTetMeshParams`` object.

        Examples
        --------
        >>> RefineTetMeshParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in RefineTetMeshParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._nisr is not None:
            json_data["nisr"] = self._nisr
        if self._niss is not None:
            json_data["niss"] = self._niss
        if self._nist is not None:
            json_data["nist"] = self._nist
        if self._tolerance is not None:
            json_data["tolerance"] = self._tolerance
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "nisr :  %s\nniss :  %s\nnist :  %s\ntolerance :  %s" % (self._nisr, self._niss, self._nist, self._tolerance)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def nisr(self) -> int:
        """Interpolation elements in the local r-direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._nisr

    @nisr.setter
    def nisr(self, value: int):
        self._nisr = value

    @property
    def niss(self) -> int:
        """Interpolation elements in the local s-direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._niss

    @niss.setter
    def niss(self, value: int):
        self._niss = value

    @property
    def nist(self) -> int:
        """Interpolation elements in the local t-direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._nist

    @nist.setter
    def nist(self, value: int):
        self._nist = value

    @property
    def tolerance(self) -> float:
        """Tolerance for boundary refinement.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._tolerance

    @tolerance.setter
    def tolerance(self, value: float):
        self._tolerance = value

class UniformSolidSplineCreationParams(CoreObject):
    """Parameters to define the uniform solid spline.

    Parameters
    ----------
    model: Model
        Model to create a ``UniformSolidSplineCreationParams`` object with default parameters.
    n_control_points_u: int, optional
        Number of control points in u direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    n_control_points_v: int, optional
        Number of control points in v direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    n_control_points_w: int, optional
        Number of control points in w direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    degree_u: int, optional
        Degree in u direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    degree_v: int, optional
        Degree in v direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    degree_w: int, optional
        Degree in w direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    json_data: dict, optional
        JSON dictionary to create a ``UniformSolidSplineCreationParams`` object with provided parameters.

    Examples
    --------
    >>> uniform_solid_spline_creation_params = prime.UniformSolidSplineCreationParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            n_control_points_u: int,
            n_control_points_v: int,
            n_control_points_w: int,
            degree_u: int,
            degree_v: int,
            degree_w: int):
        self._n_control_points_u = n_control_points_u
        self._n_control_points_v = n_control_points_v
        self._n_control_points_w = n_control_points_w
        self._degree_u = degree_u
        self._degree_v = degree_v
        self._degree_w = degree_w

    def __init__(
            self,
            model: CommunicationManager=None,
            n_control_points_u: int = None,
            n_control_points_v: int = None,
            n_control_points_w: int = None,
            degree_u: int = None,
            degree_v: int = None,
            degree_w: int = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``UniformSolidSplineCreationParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``UniformSolidSplineCreationParams`` object with default parameters.
        n_control_points_u: int, optional
            Number of control points in u direction.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        n_control_points_v: int, optional
            Number of control points in v direction.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        n_control_points_w: int, optional
            Number of control points in w direction.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        degree_u: int, optional
            Degree in u direction.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        degree_v: int, optional
            Degree in v direction.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        degree_w: int, optional
            Degree in w direction.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        json_data: dict, optional
            JSON dictionary to create a ``UniformSolidSplineCreationParams`` object with provided parameters.

        Examples
        --------
        >>> uniform_solid_spline_creation_params = prime.UniformSolidSplineCreationParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nControlPointsU"] if "nControlPointsU" in json_data else None,
                json_data["nControlPointsV"] if "nControlPointsV" in json_data else None,
                json_data["nControlPointsW"] if "nControlPointsW" in json_data else None,
                json_data["degreeU"] if "degreeU" in json_data else None,
                json_data["degreeV"] if "degreeV" in json_data else None,
                json_data["degreeW"] if "degreeW" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [n_control_points_u, n_control_points_v, n_control_points_w, degree_u, degree_v, degree_w])
            if all_field_specified:
                self.__initialize(
                    n_control_points_u,
                    n_control_points_v,
                    n_control_points_w,
                    degree_u,
                    degree_v,
                    degree_w)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "UniformSolidSplineCreationParams")
                    json_data = param_json["UniformSolidSplineCreationParams"] if "UniformSolidSplineCreationParams" in param_json else {}
                    self.__initialize(
                        n_control_points_u if n_control_points_u is not None else ( UniformSolidSplineCreationParams._default_params["n_control_points_u"] if "n_control_points_u" in UniformSolidSplineCreationParams._default_params else (json_data["nControlPointsU"] if "nControlPointsU" in json_data else None)),
                        n_control_points_v if n_control_points_v is not None else ( UniformSolidSplineCreationParams._default_params["n_control_points_v"] if "n_control_points_v" in UniformSolidSplineCreationParams._default_params else (json_data["nControlPointsV"] if "nControlPointsV" in json_data else None)),
                        n_control_points_w if n_control_points_w is not None else ( UniformSolidSplineCreationParams._default_params["n_control_points_w"] if "n_control_points_w" in UniformSolidSplineCreationParams._default_params else (json_data["nControlPointsW"] if "nControlPointsW" in json_data else None)),
                        degree_u if degree_u is not None else ( UniformSolidSplineCreationParams._default_params["degree_u"] if "degree_u" in UniformSolidSplineCreationParams._default_params else (json_data["degreeU"] if "degreeU" in json_data else None)),
                        degree_v if degree_v is not None else ( UniformSolidSplineCreationParams._default_params["degree_v"] if "degree_v" in UniformSolidSplineCreationParams._default_params else (json_data["degreeV"] if "degreeV" in json_data else None)),
                        degree_w if degree_w is not None else ( UniformSolidSplineCreationParams._default_params["degree_w"] if "degree_w" in UniformSolidSplineCreationParams._default_params else (json_data["degreeW"] if "degreeW" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_control_points_u: int = None,
            n_control_points_v: int = None,
            n_control_points_w: int = None,
            degree_u: int = None,
            degree_v: int = None,
            degree_w: int = None):
        """Set the default values of the ``UniformSolidSplineCreationParams`` object.

        Parameters
        ----------
        n_control_points_u: int, optional
            Number of control points in u direction.
        n_control_points_v: int, optional
            Number of control points in v direction.
        n_control_points_w: int, optional
            Number of control points in w direction.
        degree_u: int, optional
            Degree in u direction.
        degree_v: int, optional
            Degree in v direction.
        degree_w: int, optional
            Degree in w direction.
        """
        args = locals()
        [UniformSolidSplineCreationParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``UniformSolidSplineCreationParams`` object.

        Examples
        --------
        >>> UniformSolidSplineCreationParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in UniformSolidSplineCreationParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_control_points_u is not None:
            json_data["nControlPointsU"] = self._n_control_points_u
        if self._n_control_points_v is not None:
            json_data["nControlPointsV"] = self._n_control_points_v
        if self._n_control_points_w is not None:
            json_data["nControlPointsW"] = self._n_control_points_w
        if self._degree_u is not None:
            json_data["degreeU"] = self._degree_u
        if self._degree_v is not None:
            json_data["degreeV"] = self._degree_v
        if self._degree_w is not None:
            json_data["degreeW"] = self._degree_w
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_control_points_u :  %s\nn_control_points_v :  %s\nn_control_points_w :  %s\ndegree_u :  %s\ndegree_v :  %s\ndegree_w :  %s" % (self._n_control_points_u, self._n_control_points_v, self._n_control_points_w, self._degree_u, self._degree_v, self._degree_w)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_control_points_u(self) -> int:
        """Number of control points in u direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._n_control_points_u

    @n_control_points_u.setter
    def n_control_points_u(self, value: int):
        self._n_control_points_u = value

    @property
    def n_control_points_v(self) -> int:
        """Number of control points in v direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._n_control_points_v

    @n_control_points_v.setter
    def n_control_points_v(self, value: int):
        self._n_control_points_v = value

    @property
    def n_control_points_w(self) -> int:
        """Number of control points in w direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._n_control_points_w

    @n_control_points_w.setter
    def n_control_points_w(self, value: int):
        self._n_control_points_w = value

    @property
    def degree_u(self) -> int:
        """Degree in u direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._degree_u

    @degree_u.setter
    def degree_u(self, value: int):
        self._degree_u = value

    @property
    def degree_v(self) -> int:
        """Degree in v direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._degree_v

    @degree_v.setter
    def degree_v(self, value: int):
        self._degree_v = value

    @property
    def degree_w(self) -> int:
        """Degree in w direction.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._degree_w

    @degree_w.setter
    def degree_w(self, value: int):
        self._degree_w = value

class TrimmedSplineResults(CoreObject):
    """Results of IGA operations.

    Parameters
    ----------
    model: Model
        Model to create a ``TrimmedSplineResults`` object with default parameters.
    error_code: ErrorCode, optional
        Error code if IGA operation is unsuccessful.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    json_data: dict, optional
        JSON dictionary to create a ``TrimmedSplineResults`` object with provided parameters.

    Examples
    --------
    >>> trimmed_spline_results = prime.TrimmedSplineResults(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode):
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``TrimmedSplineResults`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``TrimmedSplineResults`` object with default parameters.
        error_code: ErrorCode, optional
            Error code if IGA operation is unsuccessful.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        json_data: dict, optional
            JSON dictionary to create a ``TrimmedSplineResults`` object with provided parameters.

        Examples
        --------
        >>> trimmed_spline_results = prime.TrimmedSplineResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [error_code])
            if all_field_specified:
                self.__initialize(
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "TrimmedSplineResults")
                    json_data = param_json["TrimmedSplineResults"] if "TrimmedSplineResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( TrimmedSplineResults._default_params["error_code"] if "error_code" in TrimmedSplineResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None):
        """Set the default values of the ``TrimmedSplineResults`` object.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code if IGA operation is unsuccessful.
        """
        args = locals()
        [TrimmedSplineResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``TrimmedSplineResults`` object.

        Examples
        --------
        >>> TrimmedSplineResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in TrimmedSplineResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s" % (self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code if IGA operation is unsuccessful.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module for mangaging file inputs and outputs."""
import json
from typing import List

# isort: split
from ansys.meshing.prime.autogen.fileio import FileIO as _FileIO

# isort: split
import ansys.meshing.prime.core.dynaexportutils as dynaexportutils
import ansys.meshing.prime.core.mapdlcdbexportutils as mapdlcdbexportutils
import ansys.meshing.prime.internals.utils as utils
from ansys.meshing.prime.autogen.fileiostructs import (
    ExportBoundaryFittedSplineParams,
    ExportFluentCaseParams,
    ExportFluentMeshingMeshParams,
    ExportLSDynaIgaKeywordFileParams,
    ExportLSDynaIGAResults,
    ExportLSDynaKeywordFileParams,
    ExportLSDynaResults,
    ExportMapdlCdbParams,
    ExportMapdlCdbResults,
    ExportSTLParams,
    FileReadParams,
    FileReadResults,
    FileWriteParams,
    FileWriteResults,
    ImportAbaqusParams,
    ImportAbaqusResults,
    ImportCadParams,
    ImportCadResults,
    ImportFluentCaseParams,
    ImportFluentCaseResults,
    ImportFluentMeshingMeshParams,
    ImportFluentMeshingMeshResults,
    ImportMapdlCdbParams,
    ImportMapdlCdbResults,
    ReadSizeFieldParams,
    SeparateBlocksFormatType,
    SizeFieldFileReadResults,
    WriteSizeFieldParams,
)
from ansys.meshing.prime.core.model import Model
from ansys.meshing.prime.params.primestructs import ErrorCode


class FileIO(_FileIO):
    """Handles reading or writing files from the disk.

    Parameters
    ----------
    model : Model
        Server model to create FileIO object.
    """

    __doc__ = _FileIO.__doc__

    def __init__(self, model: Model):
        """Initialize model and parent class."""
        self._model = model
        super().__init__(model)

    def read_pmdat(self, file_name: str, file_read_params: FileReadParams) -> FileReadResults:
        """Read a PyPrimeMesh data (PMDAT) file.

        Parameters
        ----------
        file_name : str
            Path to the data file on disk.
        file_read_params : FileReadParams
            Parameters for reading the data file.

        Returns
        -------
        FileReadResults
            Results from reading the data file.

        Notes
        -----
        This method does not support Unicode paths.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> #connect client to server and get model from it
        >>> client = prime.Client(ip="localhost", port=50060)
        >>> model = client.model
        >>> file_io = prime.FileIO(model=model)
        >>> file_read_params = prime.FileReadParams(model=model)
        >>> results = file_io.read_pmdat("/tmp/file.pmdat", file_read_params)

        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            result = super().read_pmdat(temp_file_name, file_read_params)
            if result.error_code == ErrorCode.NOERROR:
                self._model._sync_up_model()
        return result

    def write_pmdat(self, file_name: str, file_write_params: FileWriteParams) -> FileWriteResults:
        """Write a PyPrimeMesh data (PMDAT) file.

        Parameters
        ----------
        file_name : str
            Path for writing the data file on disk.
        file_write_params : FileWriteParams
            Parameters for writing the data file.

        Returns
        -------
        FileWriteResults
            Results from writing the data file.

        Examples
        --------
        >>> results = file_io.write_pmdat("/tmp/prime_mesh_data.pmdat",
                                          prime.FileWriteParams(model=model))
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().write_pmdat(temp_file_name, file_write_params)
        return result

    def import_abaqus_inp(self, file_name: str, params: ImportAbaqusParams) -> ImportAbaqusResults:
        """(BETA FEATURE) Import a Abaqus file.

        This is a beta feature to import abaqus files as dead mesh and also store
        simulation-specific information into Prime in the form of JSON documents.

        Parameters
        ----------
        file_name : str
            Name of file to import.
        params : ImportAbaqusParams
            Parameters to specify options during import.

        Returns
        -------
        ImportAbaqusResults
            Returns the results of the abaqus database import.


        Examples
        --------
        >>> results = file_io.import_abaqus(r"/tmp/file.inp")

        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            result = super().import_abaqus_inp(temp_file_name, params)
            if result.error_code == ErrorCode.NOERROR:
                self._model._sync_up_model()
        return result

    def import_fluent_meshing_size_field(self, file_name: str) -> SizeFieldFileReadResults:
        """Import a Fluent Meshing size field (SF and SF.GZ) file from disk.

        Parameters
        ----------
        file_name : str
            Path to the size field file on disk.

        Returns
        -------
        SizeFieldFileReadResults
            Results from reading the size field file.

        Notes
        -----
        This method does not support Unicode paths.

        Examples
        --------
        >>> file_io = prime.FileIO(model=model)
        >>> results = file_io.import_fluent_meshing_size_field("/tmp/my_sizefield.sf")
        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            result = super().import_fluent_meshing_size_field(temp_file_name)
        return result

    def read_size_field(
        self, file_name: str, params: ReadSizeFieldParams
    ) -> SizeFieldFileReadResults:
        """Read a PyPrimeMesh size field (PSF and PSF.GZ) file from disk.

        Parameters
        ----------
        file_name : str
            Path to the size field file on disk.
        params : ReadSizeFieldParams
            Parameters for reading size field file.

        Returns
        -------
        SizeFieldFileReadResults
            Results from reading the size field file.

        Notes
        -----
        This method does not support Unicode paths.

        Examples
        --------
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ReadSizeFieldParams(model=model)
        >>> results = file_io.read_size_field("/tmp/my_prime_sizefield.psf", params)
        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            result = super().read_size_field(temp_file_name, params)
        return result

    def write_size_field(self, file_name: str, params: WriteSizeFieldParams) -> FileWriteResults:
        """Write a PyPrimeMesh size field (PSF) file to a file on disk.

        Parameters
        ----------
        file_name : str
            Path to the size field file on disk.
        params : WriteSizeFieldParams
            Parameters for writing the size field file.

        Returns
        -------
        FileWriteResults
            Results from writing the size field file.

        Notes
        -----
        This method does not support Unicode paths.

        Examples
        --------
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.WriteSizeFieldParams(model=model)
        >>> results = file_io.write_size_field("/tmp/my_prime_sizefield.psf", params)
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().write_size_field(temp_file_name, params)
        return result

    def import_mapdl_cdb(
        self, file_name: str, params: ImportMapdlCdbParams
    ) -> ImportMapdlCdbResults:
        """Import an MAPDL CDB file.

        Parameters
        ----------
        file_name : str
            Path to the CDB file on disk.
        params : ImportMapdlCdbParams
            Parameters for importing the CDB file.

        Returns
        -------
        ImportMapdlCdbResults
            Results from importing the CDB file.

        Notes
        -----
        This method does not support Unicode paths.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> # connect client to server and get model from it
        >>> client = prime.Client(ip="localhost", port=50060)
        >>> model = client.model
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ImportMapdlCdbParams(model=model)
        >>> results = file_io.import_mapdl_cdb("/tmp/file.cdb", params)
        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            result = super().import_mapdl_cdb(temp_file_name, params)
            if result.error_code == ErrorCode.NOERROR:
                self._model._sync_up_model()
        return result

    def export_mapdl_cdb(
        self, file_name: str, params: ExportMapdlCdbParams
    ) -> ExportMapdlCdbResults:
        """Export an MAPDL CDB file.

        Parameters
        ----------
        file_name : str
            Path to the CDB file on disk.
        params : ExportMapdlCdbParams
            Parameters for exporting the CDB file.

        Returns
        -------
        ExportMapdlCdbResults
            Results from exporting the CDB file.

        Notes
        -----
        This method does not support Unicode paths.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> #connect client to server and get model from it
        >>> client = prime.Client(ip="localhost", port=50060)
        >>> model = client.model
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ExportMapdlCdbParams(model=model)
        >>> results = file_io.export_mapdl_cdb("/tmp/file.cdb", params)
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            part_id = 1
            if len(self._model.parts) > 0:
                part_id = self._model.parts[0].id
            sim_data_str = super().get_abaqus_simulation_data(part_id)
            if params.config_settings == None:
                params.config_settings = ''
            all_mat_cmds, analysis_settings = mapdlcdbexportutils.generate_mapdl_commands(
                self._model, sim_data_str, params
            )
            params.material_properties = all_mat_cmds + params.material_properties
            params.analysis_settings = analysis_settings
            result = super().export_mapdl_cdb(temp_file_name, params)
        return result

    def initialize_cdb_export_params(
        self, params: ExportMapdlCdbParams, major_version: int, minor_version: int
    ) -> ExportMapdlCdbParams:
        """
        Initialize specific CDB export parameters based on the given version.

        This function sets the separate_blocks_format_type, export_fasteners_as_swgen and
        export_rigid_bodies_as_rbgen parameters of the provided ExportMapdlCdbParams
        object based on the given major and minor version numbers.
        Other parameters remain unchanged.

        Parameters
        ----------
        params : ExportMapdlCdbParams
            The CDB export parameters object to be modified.
        major_version : int
            The major version number.
        minor_version : int
            The minor version number.

        Returns
        -------
        ExportMapdlCdbParams
            The modified CDB export parameters object.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        The version is formed as "<major_version>r<minor_version>", e.g., "24r1", "25r2".
        If the version is greater than or equal to "25r1", write_separate_blocks is set to True
        with COMPACT format, and export_fasteners_as_swgen and export_rigid_bodies_as_rbgen
        parameters are set to True. Otherwise, they are set to False.

        Examples
        --------
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ExportMapdlCdbParams()
        >>> params = file_io.initialize_cdb_export_params(params, 24, 1)
        >>> params.write_separate_blocks
        False
        >>> params.export_fasteners_as_swgen
        False
        >>> params.export_rigid_bodies_as_swgen
        False

        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ExportMapdlCdbParams()
        >>> params = file_io.initialize_cdb_export_params(params, 25, 2)
        >>> params.write_separate_blocks
        True
        >>> params.separate_blocks_format_type
        SeparateBlocksFormatType.COMPACT
        >>> params.export_fasteners_as_swgen
        True
        >>> params.export_rigid_bodies_as_swgen
        True
        """
        version = f"{major_version}r{minor_version}"
        params.write_separate_blocks = version >= "25r1"
        if version >= "25r1":
            params.separate_blocks_format_type = SeparateBlocksFormatType.COMPACT
        else:
            params.separate_blocks_format_type = SeparateBlocksFormatType.STANDARD
        params.export_fasteners_as_swgen = version >= "25r1"
        params.export_rigid_bodies_as_rbgen = version >= "25r1"
        return params

    def import_fluent_meshing_meshes(
        self,
        file_names: List[str],
        import_fluent_meshing_mesh_params: ImportFluentMeshingMeshParams,
    ) -> ImportFluentMeshingMeshResults:
        """Import Fluent Meshing's mesh (MS and MSH.GZ) files from disk.

        Parameters
        ----------
        file_names : List[str]
            List of full path for the mesh files to import.
        import_fluent_meshing_mesh_params : ImportFluentMeshingMeshParams
            Parameters for importing the mesh files.

        Returns
        -------
        ImportFluentMeshingMeshResults
            Results from importing the mesh files.


        Notes
        -----
        This method does not support Unicode paths.

        Examples
        --------
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ImportFluentMeshingMeshParams(model=model)
        >>> results = file_io.import_fluent_meshing_meshes(
                        ["/tmp/mesh.msh", "/tmp/mesh1.msh"],
                        params)

        """
        with utils.file_read_context_list(self._model, file_names) as temp_file_names:
            result = super().import_fluent_meshing_meshes(
                temp_file_names, import_fluent_meshing_mesh_params
            )
        if result.error_code == ErrorCode.NOERROR:
            self._model._sync_up_model()
        return result

    def import_fluent_case(
        self, file_name: str, import_fluent_case_params: ImportFluentCaseParams
    ) -> ImportFluentCaseResults:
        """Import a Fluent case (CAS) file from disk.

        Parameters
        ----------
        file_name : str
            Path to the case file on disk.
        import_fluent_case_params : ImportFluentCaseParams
            Parameters for importing the case file.

        Returns
        -------
        ImportFluentCaseResults
            Results from importing the case file.


        Notes
        -----
        This method does not support Unicode paths.

        Examples
        --------
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ImportFluentCaseParams(model=model)
        >>> results = file_io.import_fluent_case("/tmp/fluent.cas", params)

        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            result = super().import_fluent_case(temp_file_name, import_fluent_case_params)
            if result.error_code == ErrorCode.NOERROR:
                self._model._sync_up_model()
        return result

    def export_fluent_case(
        self, file_name: str, export_fluent_case_params: ExportFluentCaseParams
    ) -> FileWriteResults:
        """Export a Fluent case (CAS) file.

        Parameters
        ----------
        file_name : str
            Path to the case file on disk.
        export_fluent_case_params : ExportFluentCaseParams
            Parameters for exporting the case file.

        Returns
        -------
        FileWriteResults
            Results from exporting the case file.

        Examples
        --------
        >>> file_io = FileIO(model=model)
        >>> results = file_io.export_fluent_case(
                        "/tmp/fluent.cas",
                        prime.ExportFluentCaseParams(model=model))
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().export_fluent_case(temp_file_name, export_fluent_case_params)
        return result

    def export_fluent_meshing_mesh(
        self, file_name: str, export_fluent_mesh_params: ExportFluentMeshingMeshParams
    ) -> FileWriteResults:
        """Export a Fluent Meshing mesh (MSH) file.

        Parameters
        ----------
        file_name : str
            Path to the mesh file on disk.
        export_fluent_mesh_params : ExportFluentMeshingMeshParams
            Parameters for exporting the mesh file.

        Returns
        -------
        FileWriteResults
            Results from exporting the mesh file.

        Examples
        --------
        >>> results = file_io.export_fluent_meshing_mesh(
                        "/tmp/fluent_meshing.msh",
                        ExportFluentMeshingMeshParams(model=model))
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().export_fluent_meshing_mesh(temp_file_name, export_fluent_mesh_params)
        return result

    def export_lsdyna_keyword_file(
        self, file_name: str, params: ExportLSDynaKeywordFileParams
    ) -> ExportLSDynaResults:
        """Export FEA LS-DYNA Keyword file for solid, surface mesh, or both.

        Parameters
        ----------
        file_name : str
            Name of the file.
        params : ExportLSDynaKeywordFileParams
            Parameters for FEA LS-DYNA Keyword file export.

        Returns
        -------
        ExportLSDynaResults
            Returns ExportLSDynaResults.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = file_io.export_lsdyna_keyword_file(
            file_name, ExportLSDynaKeywordFileParams(model=model)
        )

        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            part_id = 1
            if len(self._model.parts) > 0:
                part_id = self._model.parts[0].id
            sim_data = json.loads(super().get_abaqus_simulation_data(part_id))
            if sim_data is not None:
                mp = dynaexportutils.MaterialProcessor(self._model, sim_data)
                all_mat_cmds = mp.get_all_material_commands()
                params.material_properties = all_mat_cmds + params.material_properties
            result = super().export_lsdyna_keyword_file(temp_file_name, params)
        return result

    def export_lsdyna_iga_keyword_file(
        self, file_name: str, params: ExportLSDynaIgaKeywordFileParams
    ) -> ExportLSDynaIGAResults:
        """Export IGA LS-DYNA Keyword file for solid, surface splines.

        Parameters
        ----------
        file_name : str
            Name of the file.
        params : ExportLSDynaIgaKeywordFileParams
            Parameters for IGA LS-DYNA Keyword file export.

        Returns
        -------
        ExportLSDynaIGAResults
            Returns ExportLSDynaIGAResults.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = file_io.export_lsdyna_iga_keyword_file(
            file_name, ExportLSDynaIgaKeywordFileParams(model=model)
        )

        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().export_lsdyna_iga_keyword_file(temp_file_name, params)
        return result

    def export_boundary_fitted_spline_kfile(
        self, file_name: str, export_params: ExportBoundaryFittedSplineParams
    ) -> FileWriteResults:
        """Export the IGA LS-DYNA keyword file for a boundary fitted spline.

        Parameters
        ----------
        file_name : str
            Name of the keyword file.
        export_params : ExportBoundaryFittedSplineParams
            Parameters for exporting the keyword file.

        Returns
        -------
        FileWriteResults
            Results from exporting the keyword file.

        Examples
        --------
        >>> results = file_io.export_boundary_fitted_spline_k_file(
                        file_name,
                        ExportBoundaryFittedSplineParams(model=model))
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().export_boundary_fitted_spline_kfile(temp_file_name, export_params)
        return result

    def import_cad(self, file_name: str, params: ImportCadParams) -> ImportCadResults:
        r"""Import a CAD file from disk.

        Supported CAD file formats on Windows are:

        \*.scdoc \*.scdocx \*.dsco \*.fmd \*.agdb \*.pmdb \*.meshdat \*.mechdat \*.dsdb \*.cmdb
        \*.sat \*.sab \*.dwg \*.dxf \*.model \*.exp \*.CATPart \*.CATProduct \*.cgr \*.3dxml
        \*.prt\* \*.asm\* \*.iges \*.igs \*.ipt \*.iam \*.jt \*.prt \*.x_t \*.x_b \*.par \*.psm
        \*.asm \*.sldprt \*.sldasm \*.step \*.stp \*.stl \*.plmxml \*.tgf

        Supported CAD file formats on Linux are:

        \*.fmd \*.agdb \*.pmdb \*.meshdat \*.mechdat \*.dsdb \*.cmdb \*.sat \*.sab
        \*.CATPart \*.CATProduct \*.iges \*.igs \*.jt \*.x_t \*.x_b \*.step \*.stp
        \*.stl \*.plmxml \*.tgf

        Refer **Reading and writing files** section in **User guide** for a
        comprehensive list of supported formats.

        Parameters
        ----------
        file_name : str
            Path to the CAD file on disk.

        params : ImportCadParams
            Parameters for importing the CAD file.

        Returns
        -------
        ImportCadResults
            Results from importing the CAD file.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> prime_client = prime.launch_prime()
        >>> model = prime_client.model
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ImportCadParams(model=model)
        >>> results = file_io.import_cad("/tmp/my_cad.x_t", params=params)

        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            import_result = super().import_cad(temp_file_name, params)
            if import_result.error_code == ErrorCode.NOERROR:
                self._model._sync_up_model()
        return import_result

    def export_stl(self, file_name: str, params: ExportSTLParams) -> FileWriteResults:
        """Export an STL file.

        Parameters
        ----------
        file_name : str
            Path to the STL file on disk.
        params : ExportSTLParams
            Parameters for exporting the STL file.

        Returns
        -------
        FileWriteResults
            Results from exporting the STL file.

        Notes
        -----
        This method does not support Unicode paths.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> model = prime.launch_prime().model
        >>> fileio = prime.FileIO(model=model)
        >>> out_file_path = r"/tmp/output.stl"
        >>> part_ids = [part.id for part in model.parts]
        >>> export_stl_params=prime.ExportSTLParams(model=model,part_ids=part_ids)
        >>> results = fileio.export_stl(out_file_path,export_stl_params)

        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().export_stl(temp_file_name, params)
        return result



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Module for PickedInfo widget."""
import os
from typing import TYPE_CHECKING

from ansys.tools.visualization_interface.backends.pyvista.widgets import PlotterWidget
from vtk import vtkPNGReader

from ansys.meshing.prime.core.mesh import DisplayMeshInfo, DisplayMeshType

if TYPE_CHECKING:
    from ansys.meshing.prime.graphics.plotter import PrimePlotter


class PickedInfo(PlotterWidget):
    """Initializes the picked information button widget.

    This widget lets you get information about the picked mesh objects.

    Parameters
    ----------
    prime_plotter : PrimePlotter
        Plotter to apply this widget to.
    """

    def __init__(self, prime_plotter: "PrimePlotter") -> None:
        """Initialize widget."""
        super().__init__(prime_plotter._backend._pl.scene)
        self.prime_plotter = prime_plotter
        self._picked_dict = self.prime_plotter._backend._picked_dict
        self._object_actors_map = self.prime_plotter._backend._object_to_actors_map
        self._button = self.prime_plotter._backend._pl.scene.add_checkbox_button_widget(
            self.callback,
            position=(5, 570),
            size=30,
            border_size=3,
            color_off="white",
            color_on="white",
        )
        self._info_actor_map = self.prime_plotter._info_actor_map

    def info_message(self, mesh_info: DisplayMeshInfo) -> str:
        """Get the information message for the selected mesh object.

        Parameters
        ----------
        mesh_info : DisplayMeshInfo
            Mesh information object to print.

        Returns
        -------
        str
            Message with the information of the selected mesh object.
        """
        mesh_type = mesh_info.display_mesh_type
        id = mesh_info.id
        part_id = mesh_info.part_id
        part_name = mesh_info.part_name
        zone_id = mesh_info.zone_id
        zone_name = mesh_info.zone_name

        if mesh_type == DisplayMeshType.TOPOFACE or mesh_type == DisplayMeshType.FACEZONELET:
            msg = "Selected FaceZonelet "
            if mesh_type is DisplayMeshType.TOPOFACE:
                msg = "Selected TopoFace "
            msg += str(id)
        elif mesh_type == DisplayMeshType.TOPOEDGE or mesh_type == DisplayMeshType.EDGEZONELET:
            msg = "Selected EdgeZonelet "
            if mesh_type is DisplayMeshType.TOPOEDGE:
                msg = "Selected TopoEdge "
            msg += str(id)
        msg += ", in Part Id : " + str(part_id) + ", Part Name : " + part_name + "\n"
        if zone_id > 0:
            msg += "Zone Id : " + str(zone_id) + ", Zone Name : " + zone_name
        return msg

    def callback(self, state: bool) -> None:
        """Define the callback function for the button widget.

        Parameters
        ----------
        state : bool
            State of the button widget.
        """
        for meshobject in list(self._picked_dict.values()):
            mesh_info = self._info_actor_map[meshobject.actor]
            print(self.info_message(mesh_info))

    def update(self) -> None:
        """Define the configuration and representation of the button widget button."""
        show_point_vr = self._button.GetRepresentation()
        show_point_icon_file = os.path.join(
            os.path.dirname(__file__), "images", "selectioninfo.png"
        )
        show_point_r = vtkPNGReader()
        show_point_r.SetFileName(show_point_icon_file)
        show_point_r.Update()
        image = show_point_r.GetOutput()
        show_point_vr.SetButtonTexture(0, image)
        show_point_vr.SetButtonTexture(1, image)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class Connect(CoreObject):
    """Connect face zonelets.

    Perform surface connection using various connect algorithms on face zonelets.

    Parameters
    ----------
    model : Model
        Server model to create Connect object.

    Notes
    -----
    Connect operations support only computational mesh, which is mesh with reasonable size changes and quality. Faceted geometry, which is STL-like mesh that can have extreme size changes and many sliver elements, not supported.
    """

    def __init__(self, model: CommunicationManager):
        """ Initialize Connect """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::Connect/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for Connect. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for Connect. """
        command_name = "PrimeMesh::Connect/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def intersect_face_zonelets(self, part_id : int, face_zonelet_ids : Iterable[int], with_face_zonelet_ids : Iterable[int], params : IntersectParams) -> ConnectResults:
        """ Perform intersection between specified face zonelets of the part with the given intersect parameters.


        Parameters
        ----------
        part_id : int
            Id of the part.
        face_zonelet_ids : Iterable[int]
            Face zonelets to be intersected.
        with_face_zonelet_ids : Iterable[int]
            Face zonelets to be intersected with.
        params : IntersectParams
            Parameters for intersection.

        Returns
        -------
        ConnectResults
            Returns the ConnectResults.


        Examples
        --------
        >>> connect = Connect(model = model)
        >>> results = connect.intersect_face_zonelets (part_id, face_zonelet_ids, with_face_zonelet_ids, params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(face_zonelet_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'face_zonelet_ids'. Valid argument type is Iterable[int].")
        if not isinstance(with_face_zonelet_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'with_face_zonelet_ids'. Valid argument type is Iterable[int].")
        if not isinstance(params, IntersectParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is IntersectParams.")
        args = {"part_id" : part_id,
        "face_zonelet_ids" : face_zonelet_ids,
        "with_face_zonelet_ids" : with_face_zonelet_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Connect/IntersectFaceZonelets"
        self._model._print_logs_before_command("intersect_face_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("intersect_face_zonelets", ConnectResults(model = self._model, json_data = result))
        return ConnectResults(model = self._model, json_data = result)

    def join_face_zonelets(self, part_id : int, face_zonelet_ids : Iterable[int], with_face_zonelet_ids : Iterable[int], params : JoinParams) -> ConnectResults:
        """ Joins a set of face zones with another set of face zones.


        Parameters
        ----------
        part_id : int
            Id of the part.
        face_zonelet_ids : Iterable[int]
            Face zonelets to be joined.
        with_face_zonelet_ids : Iterable[int]
            Face zonelets to be joined with.
        params : JoinParams
            Parameters for join.

        Returns
        -------
        ConnectResults
            Returns the ConnectResults.


        Examples
        --------
        >>> connect = Connect(model = model)
        >>> results = connect.join_face_zonelets (part_id, face_zonelet_ids, with_face_zonelet_ids, params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(face_zonelet_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'face_zonelet_ids'. Valid argument type is Iterable[int].")
        if not isinstance(with_face_zonelet_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'with_face_zonelet_ids'. Valid argument type is Iterable[int].")
        if not isinstance(params, JoinParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is JoinParams.")
        args = {"part_id" : part_id,
        "face_zonelet_ids" : face_zonelet_ids,
        "with_face_zonelet_ids" : with_face_zonelet_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Connect/JoinFaceZonelets"
        self._model._print_logs_before_command("join_face_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("join_face_zonelets", ConnectResults(model = self._model, json_data = result))
        return ConnectResults(model = self._model, json_data = result)

    def subtract_volumes(self, part_id : int, target_volumes : Iterable[int], cutter_volumes : Iterable[int], params : SubtractVolumesParams) -> SubtractVolumesResults:
        """ Subtract cutter volumes from target volumes. Volumes should be computed prior to calling this function. If multiple parts are being merged to form a single part, then volumes should be computed for each part prior to merging. Use compute_closed_volumes to do so. Volumes involved in this operation should not have self intersections. You can use SearchZoneletsBySelfIntersections to check self intersections in the volume.


        Parameters
        ----------
        part_id : int
            Id of part containing target and cutter volumes for subtract operation.
        target_volumes : Iterable[int]
            Ids of target volumes.
        cutter_volumes : Iterable[int]
            Ids of cutter volumes.
        params : SubtractVolumesParams
            Parameters to control subtraction of volumes.

        Returns
        -------
        SubtractVolumesResults
            Returns the SubtractVolumesResults.


        Examples
        --------
        >>> results = connect.subtract_volumes(part_id, target_volumes, cutter_volumes, params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(target_volumes, Iterable):
            raise TypeError("Invalid argument type passed for 'target_volumes'. Valid argument type is Iterable[int].")
        if not isinstance(cutter_volumes, Iterable):
            raise TypeError("Invalid argument type passed for 'cutter_volumes'. Valid argument type is Iterable[int].")
        if not isinstance(params, SubtractVolumesParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is SubtractVolumesParams.")
        args = {"part_id" : part_id,
        "target_volumes" : target_volumes,
        "cutter_volumes" : cutter_volumes,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Connect/SubtractVolumes"
        self._model._print_logs_before_command("subtract_volumes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("subtract_volumes", SubtractVolumesResults(model = self._model, json_data = result))
        return SubtractVolumesResults(model = self._model, json_data = result)

    def stitch_face_zonelets(self, part_id : int, face_zonelet_ids : Iterable[int], with_face_zonelet_ids : Iterable[int], params : StitchParams) -> ConnectResults:
        """ Stitches a set of face zonelets with another set of face zonelets.


        Parameters
        ----------
        part_id : int
            Id of the part.
        face_zonelet_ids : Iterable[int]
            Face zonelets to be stitched.
        with_face_zonelet_ids : Iterable[int]
            Face zonelets to be stitched with.
        params : StitchParams
            Parameters for stitch.

        Returns
        -------
        ConnectResults
            Returns the ConnectResults.


        Examples
        --------
        >>> connect = Connect(model = model)
        >>> results = connect.stitch_face_zonelets (part_id, face_zonelet_ids, with_face_zonelet_ids, stitch_params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(face_zonelet_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'face_zonelet_ids'. Valid argument type is Iterable[int].")
        if not isinstance(with_face_zonelet_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'with_face_zonelet_ids'. Valid argument type is Iterable[int].")
        if not isinstance(params, StitchParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is StitchParams.")
        args = {"part_id" : part_id,
        "face_zonelet_ids" : face_zonelet_ids,
        "with_face_zonelet_ids" : with_face_zonelet_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Connect/StitchFaceZonelets"
        self._model._print_logs_before_command("stitch_face_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("stitch_face_zonelets", ConnectResults(model = self._model, json_data = result))
        return ConnectResults(model = self._model, json_data = result)

    def merge_boundary_nodes(self, part_id : int, face_zonelet_ids : Iterable[int], with_face_zonelet_ids : Iterable[int], params : MergeBoundaryNodesParams) -> MergeBoundaryNodesResults:
        """ Merges boundary nodes of source face zonelets with boundary nodes of target face zonelets according to the provided parameters.


        Parameters
        ----------
        part_id : int
            Id of the part where merging has to take place.
        face_zonelet_ids : Iterable[int]
            Ids of the source face zonelets.
        with_face_zonelet_ids : Iterable[int]
            Ids of the target face zonelets.
        params : MergeBoundaryNodesParams
            Parameters for merging boundary nodes.

        Returns
        -------
        MergeBoundaryNodesResults
            Returns the MergeBoundaryNodesResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> connect = Connect(model = model)
        >>> results = connect.MergeBoundaryNodes(2, [2,3], [4,5], params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(face_zonelet_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'face_zonelet_ids'. Valid argument type is Iterable[int].")
        if not isinstance(with_face_zonelet_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'with_face_zonelet_ids'. Valid argument type is Iterable[int].")
        if not isinstance(params, MergeBoundaryNodesParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is MergeBoundaryNodesParams.")
        args = {"part_id" : part_id,
        "face_zonelet_ids" : face_zonelet_ids,
        "with_face_zonelet_ids" : with_face_zonelet_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Connect/MergeBoundaryNodes"
        self._model._print_beta_api_warning("merge_boundary_nodes")
        self._model._print_logs_before_command("merge_boundary_nodes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("merge_boundary_nodes", MergeBoundaryNodesResults(model = self._model, json_data = result))
        return MergeBoundaryNodesResults(model = self._model, json_data = result)

    def fuse_face_zonelets(self, part_id : int, source_face_zonelet_ids : Iterable[int], target_face_zonelet_ids : Iterable[int], params : FuseParams) -> FuseResults:
        """ Perform fuse between overlapping face zonelets within a single part. Surfaces that are fused can then be colocated, merged or removed as directed.


        Parameters
        ----------
        part_id : int
            Id of the part.
        source_face_zonelet_ids : Iterable[int]
            Ids of source face zonelets to be fused.
        target_face_zonelet_ids : Iterable[int]
            Ids of target face zonelets to be fused.
        params : FuseParams
            Parameters for fuse operation.

        Returns
        -------
        FuseResults
            Returns the FuseResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        connect = Connect(model = model)
        connect.fuse_face_zonelets(part.id, source_face_zonelet_ids, target_face_zonelet_ids, fuse_params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(source_face_zonelet_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'source_face_zonelet_ids'. Valid argument type is Iterable[int].")
        if not isinstance(target_face_zonelet_ids, Iterable):
            raise TypeError("Invalid argument type passed for 'target_face_zonelet_ids'. Valid argument type is Iterable[int].")
        if not isinstance(params, FuseParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is FuseParams.")
        args = {"part_id" : part_id,
        "source_face_zonelet_ids" : source_face_zonelet_ids,
        "target_face_zonelet_ids" : target_face_zonelet_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Connect/FuseFaceZonelets"
        self._model._print_beta_api_warning("fuse_face_zonelets")
        self._model._print_logs_before_command("fuse_face_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("fuse_face_zonelets", FuseResults(model = self._model, json_data = result))
        return FuseResults(model = self._model, json_data = result)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class Transform(CoreObject):
    """Provides functionalities to initialize and manage transformation using transformation matrix.

    Parameters
    ----------
    model : Model
        Server model to create Transform object.
    """

    def __init__(self, model: CommunicationManager):
        """ Initialize Transform """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::Transform/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for Transform. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for Transform. """
        command_name = "PrimeMesh::Transform/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def transform_zonelets(self, part_id : int, zonelets : Iterable[int], params : TransformParams) -> TransformResults:
        """ Transforms given zonelets using provided transform parameters.


        Parameters
        ----------
        part_id : int
            Part id of zonelets to be transformed.
        zonelets : Iterable[int]
            Ids of zonelets.
        params : TransformParams
            Transform parameters.

        Returns
        -------
        TransformResults
            Returns the transform results.


        Examples
        --------
        >>> params = prime.TransformParams(model=model)
        >>> # scale by a factor of 2 using a 4x4 transformation matrix
        >>> params.transformation_matrix = [
        >>>    2, 0, 0, 0,
        >>>    0, 2, 0, 0,
        >>>    0, 0, 2, 0,
        >>>    0, 0, 0, 1,
        >>> ]
        >>> part = model.get_part_by_name("part_name")
        >>> zonelets = part.get_face_zonelets()
        >>> result = prime.SurfaceUtilities(model).transform_zonelets(part.id, zonelets, params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(params, TransformParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is TransformParams.")
        args = {"part_id" : part_id,
        "zonelets" : zonelets,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Transform/TransformZonelets"
        self._model._print_logs_before_command("transform_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("transform_zonelets", TransformResults(model = self._model, json_data = result))
        return TransformResults(model = self._model, json_data = result)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class ThinVolumeControl(CoreObject):
    """ThinVolumeControl allows you to generate prisms in the space between surfaces.

    ThinVolumeControl allows you to control generation of prisms in the thin space between surfaces. Controls include setting the source face scope, target face scope and thin volume mesh parameters.

    Parameters
    ----------
    model : Model
        Server model to create ThinVolumeControl object.
    id : int
        Id of the ThinVolumeControl.
    object_id : int
        Object id of the ThinVolumeControl.
    name : str
        Name of the ThinVolumeControl.
    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize ThinVolumeControl """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def set_thin_volume_mesh_params(self, thin_volume_mesh_params : ThinVolumeMeshParams):
        """ Sets thin volume mesh parameters for thin volume control.


        Parameters
        ----------
        thin_volume_mesh_params : ThinVolumeMeshParams
            Parameters to set thin volume control growth.

        Examples
        --------
        >>> results = thin_vol_ctrl.set_thin_volume_mesh_params(ThinVolumeMeshParams(model=model))

        """
        if not isinstance(thin_volume_mesh_params, ThinVolumeMeshParams):
            raise TypeError("Invalid argument type passed for 'thin_volume_mesh_params'. Valid argument type is ThinVolumeMeshParams.")
        args = {"thin_volume_mesh_params" : thin_volume_mesh_params._jsonify()}
        command_name = "PrimeMesh::ThinVolumeControl/SetThinVolumeMeshParams"
        self._model._print_logs_before_command("set_thin_volume_mesh_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_thin_volume_mesh_params")

    def set_source_scope(self, entities : ScopeDefinition) -> SetScopeResults:
        """ Sets the source surface scope of thin volume control.


        Parameters
        ----------
        entities : ScopeDefinition
            Scope definition source face entities.

        Returns
        -------
        SetScopeResults
            Returns SetScopeResults.


        Examples
        --------
        >>> results = thin_vol_ctrl.set_source_scope(entities)

        """
        if not isinstance(entities, ScopeDefinition):
            raise TypeError("Invalid argument type passed for 'entities'. Valid argument type is ScopeDefinition.")
        args = {"entities" : entities._jsonify()}
        command_name = "PrimeMesh::ThinVolumeControl/SetSourceScope"
        self._model._print_logs_before_command("set_source_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_source_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    def set_target_scope(self, entities : ScopeDefinition) -> SetScopeResults:
        """ Sets the target surface scope of thin volume control.


        Parameters
        ----------
        entities : ScopeDefinition
            Scope definition for target face entities.

        Returns
        -------
        SetScopeResults
            Returns SetScopeResults.


        Examples
        --------
        >>> results = thin_vol_ctrl.set_target_scope(entities)

        """
        if not isinstance(entities, ScopeDefinition):
            raise TypeError("Invalid argument type passed for 'entities'. Valid argument type is ScopeDefinition.")
        args = {"entities" : entities._jsonify()}
        command_name = "PrimeMesh::ThinVolumeControl/SetTargetScope"
        self._model._print_logs_before_command("set_target_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_target_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    def set_volume_scope(self, entities : ScopeDefinition) -> SetScopeResults:
        """ Sets the volume scope of thin volume control.


        Parameters
        ----------
        entities : ScopeDefinition
            Scope definition for volume entities.

        Returns
        -------
        SetScopeResults
            Returns SetScopeResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = thin_vol_ctrl.set_volume_scope(entities)

        """
        if not isinstance(entities, ScopeDefinition):
            raise TypeError("Invalid argument type passed for 'entities'. Valid argument type is ScopeDefinition.")
        args = {"entities" : entities._jsonify()}
        command_name = "PrimeMesh::ThinVolumeControl/SetVolumeScope"
        self._model._print_beta_api_warning("set_volume_scope")
        self._model._print_logs_before_command("set_volume_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_volume_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    def get_source_scope(self) -> ScopeDefinition:
        """ Gets the source of thin volume control scope.


        Returns
        -------
        ScopeDefinition
            Returns the ScopeDefinition.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> scope = thin_vol_ctrl.get_source_scope()

        """
        args = {}
        command_name = "PrimeMesh::ThinVolumeControl/GetSourceScope"
        self._model._print_beta_api_warning("get_source_scope")
        self._model._print_logs_before_command("get_source_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_source_scope", ScopeDefinition(model = self._model, json_data = result))
        return ScopeDefinition(model = self._model, json_data = result)

    def get_target_scope(self) -> ScopeDefinition:
        """ Gets the target of thin volume control scope.


        Returns
        -------
        ScopeDefinition
            Returns the ScopeDefinition.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> scope = thin_vol_ctrl.get_target_scope()

        """
        args = {}
        command_name = "PrimeMesh::ThinVolumeControl/GetTargetScope"
        self._model._print_beta_api_warning("get_target_scope")
        self._model._print_logs_before_command("get_target_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_target_scope", ScopeDefinition(model = self._model, json_data = result))
        return ScopeDefinition(model = self._model, json_data = result)

    def get_volume_scope(self) -> ScopeDefinition:
        """ Gets the volume scope of thin volume control.


        Returns
        -------
        ScopeDefinition
            Returns the ScopeDefinition.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> scope = thin_vol_ctrl.get_volume_scope()

        """
        args = {}
        command_name = "PrimeMesh::ThinVolumeControl/GetVolumeScope"
        self._model._print_beta_api_warning("get_volume_scope")
        self._model._print_logs_before_command("get_volume_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_volume_scope", ScopeDefinition(model = self._model, json_data = result))
        return ScopeDefinition(model = self._model, json_data = result)

    @property
    def id(self):
        """ Get the id of ThinVolumeControl."""
        return self._id

    @property
    def name(self):
        """ Get the name of ThinVolumeControl."""
        return self._name
